//DevMpf.h

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
	Current Author: Marty Kraimer
	Original Author: Jim Kowalkowski (for Hideos)
	Date: 01JUL98

	This is the main base class for any EPICS MPF device support.  All
	EPICS device support that communicates with MPF tasks should be
	derived from this class.  This class hides most of the details of EPICS
	device support and the MPF (Message Passing Facility).

	Each instance of a device is bound to one MPF server, the name of the
	MPF task it is bound to is the first string in parm field of the 
	record's link.
*/

 
#ifndef DevMpfH
#define DevMpfH

class Message;
class MessageClient;
class dbCommon;
class link;

extern "C" {
#include "callback.h"
#include "dbScan.h"
#include "devSup.h"
};

#include "Message.h"
#include "ConnectMessage.h"
/*------------------------------------------------------------------------*/
/* generic structure for device support */
typedef struct {
	long        number;
	DEVSUPFUN   report;
	DEVSUPFUN   init;
	DEVSUPFUN   init_record;
	long (*get_ioint_info)(int,dbCommon*,IOSCANPVT*);
	DEVSUPFUN   read_write;
	long (*conv)(void*,int);
} MPF_DSET;

/* Macros that generate DSETs */

#define MAKE_DSET(name,init_func) extern "C" { MPF_DSET name = \
  { 5,NULL,NULL,init_func,DevMpf::ioint,DevMpf::read_write,NULL };};

#define MAKE_LINCONV_DSET(name,init_func) extern "C" { MPF_DSET name = \
 { 6,NULL,NULL,init_func,DevMpf::ioint,DevMpf::read_write,\
 DevMpf::linconv};};

/*------------------------------------------------------------------------*/
// return codes for startIO().

#define MPF_OK			0
#define MPF_NoConvert		2

class DevMpf
{
public:
    DevMpf(dbCommon*,link*,bool iointValid,int replyTimeoutSeconds = 10);

    // Following must be implemented by device support modules
    virtual long startIO(dbCommon*)=0;	// start async IO
    virtual long completeIO(dbCommon*,Message *)=0;	// end async IO
    virtual void connectIO(dbCommon*,Message *);	// connection message
    virtual long convert(dbCommon*,int pass);	// do linear conversion
    // send a message to MPF server with no reply expected
    int send(Message*); 
    // send a message to MPF server and wait for reply via completeIO
    int sendReply(Message*); 
    // This routine gets a pointer to the user portion of the parm field
    const char* getUserParm() const { return((const char*)userParm); }
    long getStatus() const { return(status);}
    int getTimeout() const {return(replyTimeout);}
    void setTimeout(int timeoutSeconds) {replyTimeout = timeoutSeconds;}
    // Following are DSET routines
    static long read_write(void*);	// generic DSET read/write routine
    static long ioint(int cmd,dbCommon*,IOSCANPVT* iopvt); // DSET i/o intr
    static long linconv(void*,int); // calls convert
private:
    static void timeoutHandler(void *pvt);
    static void messageClientCallback(Message *message,void *clientPvt);
    long status;
    dbCommon *precord;
    link  *plink;
    char* userParm;
    CALLBACK cb;
    MessageClient *pMessageClient;
    IOSCANPVT ioscanpvt;
    bool iointValid;
    connectStatus connectState;
    Message *replyMessage;
    int replyTimeout;
    bool timeout;
    bool timeoutActive;
    // disallow copy constructor, assignment operator
    DevMpf(const DevMpf &);
    DevMpf& operator=(const DevMpf&);
};

#endif // DevMpfH
