/*
    Blame:	John Winans
    Date:	4/4/95

*/
/*
This is the interface called by drvGpib.
It was originally written for hideos.
This version makes direct calls to ip support.
*/

extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <vxWorks.h>
#include <semLib.h>
#include <tickLib.h>
 
#include "drvHiDEOSGpib.h"
#include "epicsPrint.h"
}
 
#include "Gpib.h"

extern "C" {
int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
              unsigned long devAddr, unsigned long TimeOut);
void *GpibHideosInit(int BoardId, char *TaskName);
int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
              unsigned long *Actual, unsigned long devAddr, 
              unsigned long TimeOut, long Eos);
int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
              char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
              unsigned long devAddr, unsigned long TimeOut, long Eos);
int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
              unsigned long TimeOut);
};

enum gpibCommand {gpibRead, gpibWrite, gpibWriteRead, gpibCmd};

struct pvtData {
    Gpib* pGpib;
};

void *GpibHideosInit(int card, char *server)
{
    pvtData *ppvtData = new pvtData;
    ppvtData->pGpib = Gpib::findGpib(card);
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) {
	printf("GpibHideosInit could not find Gpib\n");
	return(0);
    }
    return(ppvtData);
}

int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
        unsigned long devAddr, unsigned long TimeOut)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    return(pGpib->write(Buf,(int)BufLen,(int)devAddr,(int)TimeOut));
}

int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
            unsigned long *Actual, unsigned long devAddr, 
            unsigned long TimeOut, long Eos)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    int actual;
    gpibStatus status = pGpib->read(Buf,(int)BufLen,&actual,
	(int)devAddr,(int)TimeOut,(int)Eos);
    *Actual = (unsigned long)actual;
    if(status!=gpibSuccess) return(ERROR);
    return(0);
}

int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
        char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
        unsigned long devAddr, unsigned long TimeOut, long Eos)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    int actual = pGpib->write(WBuf,(int)WBufLen,(int)devAddr,(int)TimeOut);
    if(actual!=(int)WBufLen) return(ERROR);
    gpibStatus status = pGpib->read(RBuf,(int)RBufLen,&actual,
	(int)devAddr,(int)TimeOut,(int)Eos);
    *RActual = (unsigned long)actual;
    if(status!=gpibSuccess) return(ERROR);
    return(0);
}
int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
                unsigned long TimeOut)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    Gpib *pGpib = ppvtData->pGpib;
    if(pGpib==0) return(0);
    return(pGpib->cmd(Buf,(int)BufLen,(int)TimeOut));
}
