/*
    Blame:	John Winans
    Date:	4/4/95

*/
/*
This is the interface called by drvGpib.
It was originally written for hideos. To keep compatibility 
mpf uses the same interface definition with a different implementation.
*/

extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <vxWorks.h>
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
#include "vme.h"
#include "iv.h"
 
#include "drvHiDEOSGpib.h"
#include "dbAccess.h"
#include "dbDefs.h"
#include "link.h"
#include "epicsPrint.h"
#include "dbCommon.h"
#include "stringinRecord.h"
#include "stringoutRecord.h"
#include "recSup.h"
}
 
#include "Message.h"
#include "Char8ArrayMessage.h"
#include "Int32Message.h"
#include "DevMpf.h"

extern "C" {
int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
              unsigned long devAddr, unsigned long TimeOut);
void *GpibHideosInit(int BoardId, char *TaskName);
int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
              unsigned long *Actual, unsigned long devAddr, 
              unsigned long TimeOut, long Eos);
int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
              char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
              unsigned long devAddr, unsigned long TimeOut, long Eos);
int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
              unsigned long TimeOut);
};

enum gpibCommand {gpibRead, gpibWrite, gpibWriteRead, gpibCmd};

struct pvtData {
    MessageClient *pMessageClient;
    connectStatus connectState;
    SEM_ID	sem;
    //following for each message sent to server
    bool	timeout;
    gpibCommand cmd;
    Int32Message *int32Message;
    Char8ArrayMessage *stringMessage;
};

inline void initPvtData(pvtData *p,gpibCommand cmd)
{
    p->timeout=false; p->cmd=cmd; p->int32Message=0; p->stringMessage=0;
}

static void myCallback(Message *message,void *clientPvt)
{
    pvtData *ppvtData = (pvtData *)clientPvt;
    
    switch(message->getType()) {
    case messageTypeConnect: {
	    ConnectMessage *pConnectMessage = (ConnectMessage *)message;
	    ppvtData->connectState = pConnectMessage->status;
	    delete pConnectMessage;
	    semGive(ppvtData->sem);
	}
	break;
    case messageTypeInt32:
	ppvtData->int32Message = (Int32Message *)message;
	semGive(ppvtData->sem);
	break;
    case messageTypeChar8Array:
	ppvtData->stringMessage = (Char8ArrayMessage *)message;
	semGive(ppvtData->sem);
	break;
    default:
	printf("GpibHideos: got illegal response message\n");
	break;;
    }
}

static void timeoutHandler(void *pvt)
{
    pvtData *ppvtData = (pvtData *)pvt;
    ppvtData->timeout = true;
    semGive(ppvtData->sem);
}

void *GpibHideosInit(int location, char *server)
{
    pvtData *ppvtData = new pvtData;
    ::memset(ppvtData,0,sizeof(pvtData));
    ppvtData->connectState = connectNo;
    ppvtData->sem  = semBCreate(SEM_Q_PRIORITY,SEM_EMPTY);
    MessageClient *pMessageClient = new MessageClient(myCallback,ppvtData);
    ppvtData->pMessageClient = pMessageClient;
    int status = pMessageClient->bind(server,location);
    if(status) {
        printf("GpibHideosInit: bind returned %d\n",status);
    }
    return(ppvtData);
}

int GpibHideosWrite(void *td, char *Buf, unsigned long BufLen, 
        unsigned long devAddr, unsigned long TimeOut)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState == connectNo) return(ERROR);
    initPvtData(ppvtData,gpibWrite);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Char8ArrayMessage *stringMessage = new Char8ArrayMessage;
    stringMessage->timeout = (int32)TimeOut;;
    stringMessage->cmd = gpibWrite;
    stringMessage->address = (int32)devAddr;
    stringMessage->allocValue(BufLen);
    stringMessage->setSize(BufLen);
    ::memcpy(stringMessage->value,Buf,BufLen);
    int status = pMessageClient->sendTimeout(stringMessage,
        TimeOut,timeoutHandler,(void*)ppvtData);
    if(status) return(ERROR);
    semTake(ppvtData->sem,WAIT_FOREVER);
    if(ppvtData->timeout) return(ERROR);
    Int32Message *int32Message = ppvtData->int32Message;
    status = int32Message->status;
    delete int32Message;
    return(status);
}

int GpibHideosRead(void *td, char *Buf, unsigned long BufLen, 
            unsigned long *Actual, unsigned long devAddr, 
            unsigned long TimeOut, long Eos)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState == connectNo) return(ERROR);
    initPvtData(ppvtData,gpibRead);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Int32Message *int32Message = new Int32Message;
    int32Message->timeout = (int32)TimeOut;;
    int32Message->cmd = gpibRead;
    int32Message->address = (int32)devAddr;
    int32Message->extra = (int32)Eos;
    int32Message->value = (int32)BufLen;
    int status = pMessageClient->sendTimeout(int32Message,
        TimeOut,timeoutHandler,(void*)ppvtData);
    if(status) return(ERROR);
    semTake(ppvtData->sem,WAIT_FOREVER);
    if(ppvtData->timeout) return(ERROR);
    Char8ArrayMessage *stringMessage = ppvtData->stringMessage;
    *Actual = (unsigned long)stringMessage->getSize();
    ::memcpy(Buf,stringMessage->value,stringMessage->getSize());
    status = stringMessage->status;
    delete stringMessage;
    if(status) return(ERROR);
    return(0);
}

int GpibHideosWriteRead(void *td, char *WBuf, unsigned long WBufLen, 
        char *RBuf, unsigned long RBufLen, unsigned long *RActual, 
        unsigned long devAddr, unsigned long TimeOut, long Eos)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState == connectNo) return(ERROR);
    initPvtData(ppvtData,gpibWriteRead);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Int32Message *int32Message = new Int32Message;
    int32Message->timeout = (int32)TimeOut;;
    int32Message->cmd = gpibWriteRead;
    int32Message->address = (int32)devAddr;
    int32Message->extra = (int32)Eos;
    int32Message->value = (int32)RBufLen;
    int status = pMessageClient->send(int32Message);
    if(status) return(ERROR);
    Char8ArrayMessage *stringMessage = new Char8ArrayMessage;
    stringMessage->timeout = (int32)TimeOut;;
    stringMessage->cmd = gpibWriteRead;
    stringMessage->address = (int32)devAddr;
    stringMessage->allocValue(WBufLen);
    stringMessage->setSize(WBufLen);
    ::memcpy(stringMessage->value,WBuf,WBufLen);
    status = pMessageClient->sendTimeout(stringMessage,
        TimeOut,timeoutHandler,(void*)ppvtData);
    if(status) return(ERROR);
    semTake(ppvtData->sem,WAIT_FOREVER);
    if(ppvtData->timeout) return(ERROR);
    stringMessage = ppvtData->stringMessage;
    *RActual = (unsigned long)stringMessage->getSize();
    ::memcpy(RBuf,stringMessage->value,stringMessage->getSize());
    status = stringMessage->status;
    delete stringMessage;
    if(status) return(ERROR);
    return(0);
}
int GpibHideosWriteCmd(void *td, char *Buf, unsigned long BufLen, 
                unsigned long TimeOut)
{
    TimeOut /= 60; //convert to seconds
    pvtData *ppvtData = (pvtData *)td;
    if(ppvtData->connectState == connectNo) return(ERROR);
    initPvtData(ppvtData,gpibCmd);
    MessageClient *pMessageClient = ppvtData->pMessageClient;
    Char8ArrayMessage *stringMessage = new Char8ArrayMessage;
    stringMessage->timeout = (int32)TimeOut;;
    stringMessage->cmd = gpibCmd;
    stringMessage->allocValue(BufLen);
    stringMessage->setSize(BufLen);
    ::memcpy(stringMessage->value,Buf,BufLen);
    int status = pMessageClient->sendTimeout(stringMessage,
        TimeOut,timeoutHandler,(void*)ppvtData);
    if(status) return(ERROR);
    semTake(ppvtData->sem,WAIT_FOREVER);
    if(ppvtData->timeout) return(ERROR);
    Int32Message *int32Message = ppvtData->int32Message;
    status = int32Message->status;
    delete int32Message;
    return(status);
}
