//devStringMpf.cc
extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
 
#include "dbAccess.h"
#include "dbDefs.h"
#include "link.h"
#include "epicsPrint.h"
#include "dbCommon.h"
#include "stringinRecord.h"
#include "stringoutRecord.h"
#include "recSup.h"
}

#include "Message.h"
#include "Char8ArrayMessage.h"
#include "DevMpf.h"
#include "serialServer.h"

/*
   INP has form Cx Sx @server,INstringPV
   INstringPV is sent and response is placed in VAL
*/

class DevStringin : public DevMpf
{
public:
    DevStringin(dbCommon*,DBLINK*);
    long startIO(dbCommon* pr);
    long completeIO(dbCommon* pr,Message* m);
    static long dev_init(void*);
private:
    DBADDR db;
    char buffer[100];
};

MAKE_DSET(devSiMpf,DevStringin::dev_init)

long DevStringin::dev_init(void* v)
{
    stringinRecord* pr = (stringinRecord*)v;
    new DevStringin((dbCommon*)pr,&(pr->inp));
    return(0);
}

DevStringin::DevStringin(dbCommon* pr,DBLINK* l) : DevMpf(pr,l,false)
{
    const char* pv = getUserParm();
    if(pv==0) {
	epicsPrintf("%s DevStringin Illegal INP field\n",pr->name);
        pr->pact = TRUE;
        return;
    }
    if(dbNameToAddr(pv,&db)) {
	epicsPrintf("%s DevStringin Can't locate PC %s\n",pr->name,pv);
        pr->pact = TRUE;
        return;
    }
}

long DevStringin::startIO(dbCommon *pr)
{
    stringinRecord *psi = (stringinRecord *)pr;
    long status = dbGet(&db,DBR_STRING,buffer,0,0,0);
    if(status) {
	epicsPrintf("%s dbGet for input string failed\n");
	recGblSetSevr(psi,READ_ALARM,INVALID_ALARM);
	return(0);
    }
    Char8ArrayMessage *message = new Char8ArrayMessage;
    int len = ::strlen(buffer) + 1;
    message->allocValue(len);
    message->setSize(len);
    ::strcpy(message->value,buffer);
    message->cmd = cmdWriteRead;
    message->timeout = 5;
    return(sendReply(message));
}

long DevStringin::completeIO(dbCommon* pr,Message* pmessage)
{
    stringinRecord* psi = (stringinRecord*)pr;
    if((pmessage->getType()) != messageTypeChar8Array) {
	if(pmessage->getType() != messageTypeInt32) 
	    epicsPrintf("%s DevStringin::completeIO illegal message type %d\n",
	    psi->name,pmessage->getType());
	recGblSetSevr(psi,READ_ALARM,INVALID_ALARM);
    } else {
        Char8ArrayMessage *pChar8ArrayMessage = (Char8ArrayMessage *)pmessage;
        ::strcpy(psi->val,pChar8ArrayMessage->value);
        psi->udf = 0;
    }
    delete pmessage;
    return(MPF_OK);
}
