// Gpib.cc

// Author: Marty Kraimer
// Date: July 1998

extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
}
 
#include "WatchDog.h"
#include "DLList.h"
#include "Gpib.h"
DLList *Gpib::pCardList = 0;


void Gpib::timoutHandler(void* v)
{
    Gpib* pGpib = (Gpib*)v;

    pGpib->ioStatus = gpibTimeout;
    semGive(pGpib->ioComplete);
}

void Gpib::startIoTimeout(int seconds)
{
    // Make sure there is no lingering semGive
    semTake(ioComplete,NO_WAIT);
    pwg->start(seconds,timoutHandler,this);
}
 
void Gpib::waitIo()
{
    semTake(ioComplete,WAIT_FOREVER);
    pwg->cancel();
}

void Gpib::waitIo(int seconds)
{
    pwg->start(seconds,timoutHandler,this);
    waitIo();
}

Gpib *Gpib::findGpib(int card)
{
    if(pCardList==0) pCardList = new DLList();
    DLNode *node = pCardList->getFirst();
    while(node) {
        Gpib *pcard = (Gpib *)node->obj;
        if(pcard->card == card) {
            return(pcard);
        }
        node = pCardList->getNext(node);
    }
    return(0);
}

Gpib::Gpib(int c)
: card(c), ioStatus(gpibSuccess)
{
    Gpib *pGpib = findGpib(card);
    if(pGpib) {
        printf("Gpib::Gpib card %d already initialized\n",card);
        return(this);
    }
    DLNode *node = DLList::allocNode();
    node->obj = (Gpib *)this;
    pCardList->addEnd(node);
    ioComplete = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    pwg = new WatchDog;
}
