// GpibGsTi9914.cc

// Author: John Winans
// Date: 12/94
// Change from Hideos to MPF: Marty Kraimer July 1998

extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
#include <intLib.h>
}
 
#include "WatchDog.h"
#include "DLList.h"
#include "IpModule.h"
#include "Gpib.h"


/*
 * Support for the IP-488 (Green Springs GPIB IP using the ti9914)
 *			John Winans 12/94
 */


typedef volatile struct
{
  unsigned char spare0;
  unsigned char IntReg0;
  unsigned char spare2;
  unsigned char IntReg1;
  unsigned char spare4;
  unsigned char AddressStatus;
  unsigned char spare6;
  unsigned char CmdStatus;
  unsigned char spare8;
  unsigned char Address;
  unsigned char sparea;
  unsigned char SerialPoll;
  unsigned char sparec;
  unsigned char CommandPass;
  unsigned char sparee;
  unsigned char Data;
  unsigned char spare10;
  unsigned char AddressSwitch;
  unsigned char spare12;
  unsigned char VectorRegister;
} Ip488RegStruct;

class GpibGS : public Gpib
{
public:
    GpibGS(int card);
    virtual int cmd(const char *buf, int cnt, int time_limit);
    virtual int write(const char *buf, int cnt,
	int address, int time_limit);
    virtual gpibStatus read(char *buf, int cnt, int *actual,
	int address, int time_limit, int eos);
private:
    static void* ifunc(void*);
    void sicGpib(void);
    int cmdGpib(const char *buf, int cnt, int time_limit);
    int writeGpib(const char *buf, int cnt, bool raw_flag, int time_limit);
    int readGpib(char *buf, int cnt, int time_limit, int eos);
    
    IpModule* ip;			// my ipmodule descriptor
    Ip488RegStruct	*regs;		// hardware registerz
    volatile unsigned char CopyIsr0; // can be altered in the IRQ handler
    volatile unsigned char CopyIsr1; // can be altered in the IRQ handler
    volatile enum	{ACTIVE, INACTIVE} ReadHoldoff;
    volatile long	TransferSize;
    volatile char	*TransferBuffer;
    volatile enum	{YES, NO} TransferEoiOnLast;
    volatile enum	{IP488_READ, IP488_WRITE, IP488_IDLE} TransferState;
    volatile long   TransferEos;

};

#define  TMOTIMELIMIT   100000 /* time to wait r/w accesses to same   */
 
// Hardware register bit definitions 
//      Name            Bit(s)       register

#if 0
#define adredpa         (1<<7)     /*  adr   */
#endif

#define isr0END         (1<<3)		// EOI present on bus
#define isr0BO          (1<<4)		// Ready to write a byte
#define isr0BI          (1<<5)		// Ready to read byte

#define isr1SRQ         (1<<1)		// SRQ on bus is active
#define isr1ERR		(1<<6)		// handshake error (no listener)

#define IP488_CIC_ADDR  0       // Primary address of the TI9914 controller
 
/* TI9914 Auxiliary Commands */
 
#define IP488_AUXswrstC    0x00    /* Software reset CLEAR              */
#define IP488_AUXswrstS    0x80    /* Software reset SET                */
#define IP488_AUXdacrC     0x01    /* Release DAC holdoff CLEAR         */
#define IP488_AUXdacrS     0x81    /* Release DAC holdoff SET           */
#define IP488_AUXrhdf      0x02    /* Release RFD holdoff               */
#define IP488_AUXhdfaC     0x03    /* Holdoff on all data CLEAR         */
#define IP488_AUXhdfaS     0x83    /* Holdoff on all data SET           */
#define IP488_AUXhdfeC     0x04    /* Holdoff on EOI only CLEAR         */
#define IP488_AUXhdfeS     0x84    /* Holdoff on EOI only SET           */
#define IP488_AUXnbaf      0x05    /* New byte available false          */
#define IP488_AUXfgetC     0x06    /* Force group execute trigger CLEAR */
#define IP488_AUXfgetS     0x86    /* Force group execute trigger SET   */
#define IP488_AUXrtlC      0x07    /* Return to local CLEAR             */
#define IP488_AUXrtlS      0x87    /* Return to local SET               */
#define IP488_AUXfeoi      0x08    /* Send EOI with next byte           */
#define IP488_AUXlonC      0x09    /* Listen only CLEAR                 */
#define IP488_AUXlonS      0x89    /* Listen only SET                   */
#define IP488_AUXtonC      0x0A    /* Talk only CLEAR                   */
#define IP488_AUXtonS      0x8A    /* Talk only SET                     */
#define IP488_AUXgts       0x0B    /* Go to standby                     */
#define IP488_AUXtca       0x0C    /* Take control asynchronously       */
#define IP488_AUXtcs       0x0D    /* Take control synchronously        */
#define IP488_AUXrppC      0x0E    /* Request parallel poll CLEAR       */
#define IP488_AUXrppS      0x8E    /* Request parallel poll SET         */
#define IP488_AUXsicC      0x0F    /* Send interface clear CLEAR        */
#define IP488_AUXsicS      0x8F    /* Send interface clear SET          */
#define IP488_AUXsreC      0x10    /* Send remote enable CLEAR          */
#define IP488_AUXsreS      0x90    /* Send remote enable SET            */
#define IP488_AUXrqc       0x11    /* Request Control                   */
#define IP488_AUXrlc       0x12    /* Release Control                   */
#define IP488_AUXdaiC      0x13    /* Disable all interrupts CLEAR      */
#define IP488_AUXdaiS      0x93    /* Disable all interrupts SET        */
#define IP488_AUXpts       0x14    /* Pass through next secondary       */
#define IP488_AUXstdlC     0x15    /* Short TI settling time CLEAR      */
#define IP488_AUXstdlS     0x95    /* Short TI settling time SET        */
#define IP488_AUXshdwC     0x16    /* Shadow handshake CLEAR            */
#define IP488_AUXshdwS     0x96    /* Shadow handshake SET              */
#define IP488_AUXvstdlC    0x17    /* Very short T1 delay CLEAR         */
#define IP488_AUXvstdlS    0x97    /* Very short T1 delay SET           */
#define IP488_AUXrsv2C     0x18    /* Request Service Bit 2 CLEAR       */
#define IP488_AUXrsv2S     0x98    /* Request Service Bit 2 SET         */

static void wasteTime()
{
   volatile int value = 0;
   // delay for >= 600us
   for(int i= 0; i<100000; i++) value++;		// delay for >= 600us
}

int initGpibGsTi9914(int card)
{
    new GpibGS(card);
    return(0);
}

GpibGS::GpibGS(int c) : Gpib(c)
{
    IpModule *pmodule = IpModule::findIpModule(card);
    if (pmodule == 0) {
        printf("OctalUART::OctalUART IpModule card %d not found\n", card);
        return(this);
    }
    ip = pmodule;
    // get a block of interrupt vectors
    int	int_vec=ip->getIntVector(0);
    int_vec=ip->getIntVector(1);
    // Find the register map of the IP card
    regs=(Ip488RegStruct *)ip->getIoSpace();
    ip->intDisable(0);
    // configure the chip in here
    regs->VectorRegister = int_vec;
    ip->connectInterrupt(0,(interruptHandler)ifunc,(void *)this);
    ip->connectInterrupt(1,(interruptHandler)ifunc,(void *)this);
    // Issue a software reset to the GPIB controller chip
    regs->CmdStatus = IP488_AUXswrstS;	// assert swrst

    // Set the ti9914's address to use while it is CIC
    regs->Address = IP488_CIC_ADDR;

    // Toss interrupt status values before we release the reset
    CopyIsr0 = regs->IntReg0;   
    CopyIsr1 = regs->IntReg1;

    regs->CmdStatus = IP488_AUXswrstC;	// CLEAR swrst

    // Enable IRQs that we care about
    //regs->IntReg0 = isr0END|isr0BO|isr0BI;
    //regs->IntReg1 = isr1SRQ|isr1ERR;

    regs->IntReg0 = isr0BO|isr0BI;
    regs->IntReg1 = isr1ERR;

    wasteTime();
    sicGpib();				// assert IFC
    wasteTime();
    regs->CmdStatus = IP488_AUXsreS;	// assert REN
    wasteTime();

    regs->CmdStatus = IP488_AUXdaiC;	// Clear dai

    ReadHoldoff = INACTIVE;

    // Prime our copies of these registers
    CopyIsr0 = regs->IntReg0;   
    CopyIsr1 = regs->IntReg1;
    CopyIsr0 = CopyIsr0 | isr0BO;


    ip->intClear(0);
    ip->intEnable(0);
}

int GpibGS::cmd(const char *buf, int cnt, int time_limit)
{
    return(cmdGpib(buf,cnt,time_limit));
}

int GpibGS::write(const char *buf, int cnt, int addr, int time_limit)
{
    char address = (char)addr + 0x20;
    if(cmdGpib("_?", 2, time_limit) != 2) return(-1);
    if(cmdGpib(&address, 1, time_limit) != 1) return(-1);
    int actual = writeGpib(buf,cnt,0,time_limit);
    if(cmdGpib("_?", 2, time_limit) != 2) return(-1);
    return(actual);
}

gpibStatus GpibGS::read(char *buf, int cnt, int *actual,
    int addr, int time_limit, int eos)
{
    char address = (char)addr + 0x40;
    if(cmdGpib("_?", 2, time_limit) != 2) return(gpibTimeout);
    if(cmdGpib(&address, 1, time_limit)!=1) return(ioStatus);
    *actual = readGpib(buf,cnt,time_limit,eos);
    if(cmdGpib("_?", 2, time_limit) != 2) return(gpibTimeout);
    return(gpibSuccess);
}

// IRQ
// enters here when tranmit buffer empty
void* GpibGS::ifunc(void* v)
{
    GpibGS *pGpib = (GpibGS *)v;
    int message_complete = 0;

    if(pGpib->ip) pGpib->ip->intClear(0);

    pGpib->CopyIsr0 |= pGpib->regs->IntReg0;
    pGpib->CopyIsr1 |= pGpib->regs->IntReg1;

    //
    // Can be in reading, writing, command, or idle mode at this point
    //
    switch (pGpib->TransferState)
    {
    case IP488_READ:
        if (pGpib->CopyIsr0 & isr0BI)
        { // We are ready to read a byte now
            if (pGpib->TransferSize == 0)
            { // We are out of buffer space
                pGpib->TransferState = IP488_IDLE;
                pGpib->ioStatus = gpibFailure;	// buffer overflow
		semGive(pGpib->ioComplete);
                return NULL;
            }
            else if (pGpib->TransferSize == 1)
            { // This is the last character which will fit in
                // the buffer, set hold off
                pGpib->regs->CmdStatus = IP488_AUXhdfaS;    // Holdoff on now
                *pGpib->TransferBuffer = pGpib->regs->Data;
                pGpib->regs->CmdStatus = IP488_AUXhdfaC;
                message_complete = 1;
            }
            else if (pGpib->TransferEos != -1 ) 
            { // Check for EOS character - must read with holdoff or we
                // will lose the character after the EOS
                pGpib->regs->CmdStatus = IP488_AUXhdfaS;    // Holdoff on now
                *pGpib->TransferBuffer = pGpib->regs->Data;
                pGpib->regs->CmdStatus = IP488_AUXhdfaC;
                if (*pGpib->TransferBuffer == pGpib->TransferEos)
                    // This is EOS character, message complete
                    message_complete = 1;
                else
                    // Not EOS character, release read holdoff
                    pGpib->regs->CmdStatus = IP488_AUXrhdf;
            }
            else
                *pGpib->TransferBuffer = pGpib->regs->Data;


            --(pGpib->TransferSize);
            ++(pGpib->TransferBuffer);
            pGpib->CopyIsr0 &= ~isr0BI;
        }
        if (pGpib->CopyIsr0 & isr0END)
        { // EOI was asserted for this character
            pGpib->CopyIsr0 &= ~isr0END;
                        message_complete = 1;
        }
        if (message_complete)
        {
            pGpib->TransferState = IP488_IDLE;
	    pGpib->ioStatus = gpibSuccess;
	    semGive(pGpib->ioComplete);
            return NULL;
        }
        break;
    case IP488_WRITE:
        // If data transmitter is ready, send next byte
        if (pGpib->CopyIsr1 & isr1ERR)
        { // Handshake failure -- we ain't got a listener
            pGpib->CopyIsr1 &= ~isr1ERR;		// Clear error status
            pGpib->regs->CmdStatus = IP488_AUXnbaf;	// Loose that last byte
            ++(pGpib->TransferSize);			// Clean up
            --(pGpib->TransferBuffer);			// Clean up
            printf("Handshake failure\n");
            pGpib->TransferState = IP488_IDLE;
	    pGpib->ioStatus = gpibFailure;
	    semGive(pGpib->ioComplete);
            return NULL;
        }
        if (pGpib->CopyIsr0 & isr0BO)
        { // We are ready to send a byte now
            if (pGpib->TransferSize == 0)
            { // Send message to indicate transfer completion
                pGpib->TransferState = IP488_IDLE;
	        pGpib->ioStatus = gpibSuccess;
	        semGive(pGpib->ioComplete);
                return NULL;
            }

            if ((pGpib->TransferSize == 1) && (pGpib->TransferEoiOnLast == YES))
                pGpib->regs->CmdStatus = IP488_AUXfeoi;
            
            pGpib->regs->Data = *pGpib->TransferBuffer;
            --(pGpib->TransferSize);
            ++(pGpib->TransferBuffer);

            pGpib->CopyIsr0 &= ~isr0BO;
        }
        break;
    case IP488_IDLE:
        break;
    }

    return NULL;
}

void GpibGS::sicGpib(void)
{
    regs->CmdStatus = IP488_AUXsicS;	// assert the IFC line
    // BUG -- change to use a delay timer!
    // BUG -- be careful if using timer in constructor... dispatcher yet!
    wasteTime();
    regs->CmdStatus = IP488_AUXsicC;	// release the IFC line
}

// Write a string with the ATN line active
int GpibGS::cmdGpib(const char *buf, int cnt, int time_limit)
{

    regs->CmdStatus = IP488_AUXtca;	// Take control of GPIB

    if (cnt == 0)
        return(gpibSuccess);

    TransferEoiOnLast = NO;
    TransferSize = cnt;
    TransferBuffer = buf;
    TransferState = IP488_WRITE;

    // Send the first byte manually
    int lockKey = ::intLock();
    ifunc(this);
    ::intUnlock(lockKey);
    waitIo(time_limit);
    if(ioStatus==gpibTimeout) sicGpib();
    return(cnt - TransferSize);
}

// Read a string from the bus.  Stop on EOI, EOS, or buffer full
int GpibGS::readGpib(char *buf, int cnt, int time_limit, int eos)
{
    if(cnt == 0)
        return (0);

    regs->CmdStatus = IP488_AUXgts;             // If CAC, go into standby
    regs->CmdStatus = IP488_AUXlonS;		// force into listener state

    // If are currently holding off on the receiver, finish it
    if (ReadHoldoff == ACTIVE)
        regs->CmdStatus = IP488_AUXrhdf;

    ReadHoldoff = ACTIVE;
    regs->CmdStatus = IP488_AUXhdfeS;		// Holdoff on EOI

    TransferSize = cnt;
    TransferBuffer = buf;
    TransferState = IP488_READ;
    TransferEos = eos;

    int lockKey = ::intLock();
    ifunc(this);
    ::intUnlock(lockKey);

    waitIo(time_limit);
    if(ioStatus==gpibTimeout) sicGpib();
    regs->CmdStatus = IP488_AUXlonC;	// force outta listener state
    return(cnt - TransferSize);
}

// Write a string with the ATN line inactive.
int GpibGS::writeGpib(const char *buf, int cnt, bool raw_flag, int time_limit)
{
    if (cnt == 0)
        return (0);

    regs->CmdStatus = IP488_AUXgts;		// If CAC, go into standby
    regs->CmdStatus = IP488_AUXtonS;		// Force talker on
    if(raw_flag) 
	TransferEoiOnLast = NO;
    else
	TransferEoiOnLast = YES;

    TransferSize = cnt;
    TransferBuffer = buf;
    TransferState = IP488_WRITE;

    // Send the first byte manually
    int lockKey = ::intLock();
    ifunc(this);
    ::intUnlock(lockKey);

    waitIo(time_limit);
    if(ioStatus==gpibTimeout) sicGpib();
    regs->CmdStatus = IP488_AUXtonC;
    return(cnt - TransferSize);

}
