// IndustryPackCarrier.h

/********************COPYRIGHT NOTIFICATION**********************************
  This software was developed under a United States Government license
  described on the COPYRIGHT_UniversityOfChicago file included as part
  of this distribution.
  ***************************************************************************
  */
/*
  Current Author: Thomas Coleman
  Original Author: Jim Kowalkowski (for Hideos)
  Date: 09JUL98
  */

#ifndef IndustryPackCarrierH
#define IndustryPackCarrierH

extern "C"
{
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <vxLib.h>
#include <intLib.h>
}

#include "DLList.h"

struct ip_id_prom
{
  char dum0;
  unsigned char ascii1;
  char dum1;
  unsigned char ascii2;
  char dum2;
  unsigned char ascii3;
  char dum3;
  unsigned char ascii4;
  char dum4;
  unsigned char manufacturer;
  char dum5;
  unsigned char model;
  char dum6;
  unsigned char revision;
  char dum7;
  unsigned char reserved;
  char dum8;
  unsigned char driverLow;
  char dum9;
  unsigned char driverHigh;
  char dum10;
  unsigned char numBytesUsed;
  
  unsigned char userStuff[64-sizeof(unsigned short)*11];
};
typedef struct ip_id_prom IP_ID_PROM;

class IndustryPackSite;

class IndustryPackCarrier
{
 public:
  IndustryPackCarrier();
  int registerName(const char *carrierName);
  static IndustryPackCarrier *find(const char *carrierName);
  IndustryPackSite *findSite(const char *siteName);
  IndustryPackSite *addSite(IndustryPackSite *pSite);
  const char *getCarrierName();
  virtual volatile void *allocMemMapIP(int size) = 0;
 private:
  const char *carrierName;
  DLList *pIpSiteList;
};

class IndustryPackSite
{
 public:
  IndustryPackSite();
  const char *getSiteName();
  virtual volatile void* allocMemMapIP(int size) = 0;
  virtual volatile IP_ID_PROM* getMemBaseID() = 0;
  virtual volatile void* getMemBaseIO() = 0;
  virtual volatile void* getMemBaseIP() = 0;
  virtual int intConfig(int num) = 0;
  virtual void intEnable(int num) = 0;
  virtual void intDisable(int num) = 0;
 protected:
  const char *siteName;
  IndustryPackCarrier *pIpCarrier;
 private:
};

#endif /* IndustryPackCarrierH */

