// IndustryPackMVME162.cc

/********************COPYRIGHT NOTIFICATION**********************************
  This software was developed under a United States Government license
  described on the COPYRIGHT_UniversityOfChicago file included as part
  of this distribution.
  ***************************************************************************
  */
/*
  Current Author: Thomas Coleman
  Original Author: Jim Kowalkowski (for Hideos)
  Date: 09JUL98
  */

extern "C"
{
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <vxLib.h>
#include <intLib.h>
}

#include "DLList.h"
#include "IndustryPackCarrier.h"


// MVME162 Local Bus I/O Devices Memory Map

#define IPIC_MAX_SITES 4

#define IPIC_CNTL_STAT_REGS_MAP 0xfffbc000

#define IPIC_IP_A_NAME "IP_a"
#define IPIC_IP_B_NAME "IP_b"
#define IPIC_IP_C_NAME "IP_c"
#define IPIC_IP_D_NAME "IP_d"

#define IPIC_IP_AB_NAME "IP_ab"
#define IPIC_IP_CD_NAME "IP_cd"

#define IPIC_IP_A_ID_MAP 0xfff58080
#define IPIC_IP_B_ID_MAP 0xfff58180
#define IPIC_IP_C_ID_MAP 0xfff58280
#define IPIC_IP_D_ID_MAP 0xfff58380

#define IPIC_IP_A_IO_MAP 0xfff58000
#define IPIC_IP_B_IO_MAP 0xfff58100
#define IPIC_IP_C_IO_MAP 0xfff58200
#define IPIC_IP_D_IO_MAP 0xfff58300

#define IPIC_IP_AB_IO_MAP 0xfff58400
#define IPIC_IP_CD_IO_MAP 0xfff58500

//

#define CNTL_MEN 0x01
#define CNTL_ERR 0x80
#define CNTL_WIDTH 0x0c /* 00=32bit, 01=8bit, 10=16bits, 11=bad */
#define CNTL_WIDTH_32 0x00 /* 00=32bit, 01=8bit, 10=16bits, 11=bad */
#define CNTL_WIDTH_8 0x04 /* 00=32bit, 01=8bit, 10=16bits, 11=bad */
#define CNTL_WIDTH_16 0x08 /* 00=32bit, 01=8bit, 10=16bits, 11=bad */
#define CNTL_RT 0x30 /* 00=0us, 01=2us, 10=4us, 11=8us */
#define CNTL_RT_0 0x00 /* 00=0us, 01=2us, 10=4us, 11=8us */
#define CNTL_RT_2 0x10 /* 00=0us, 01=2us, 10=4us, 11=8us */
#define CNTL_RT_4 0x20 /* 00=0us, 01=2us, 10=4us, 11=8us */
#define CNTL_RT_8 0x30 /* 00=0us, 01=2us, 10=4us, 11=8us */

#define IP_RESET 0x01

#define IP_INT_LEVEL 0x07 /* int level here 0-7 */
#define IP_INT_ICLR 0x08 /* int clear for edge sens mode */
#define IP_INT_IEN 0x10 /* int enable */
#define IP_INT_INT 0x20 /* int being generated */
#define IP_INT_EL 0x40 /* edge=1, level=0 */
#define IP_INT_PLTY 0x80 /* falling edge/low level=0,rising edge/high level=1 */

struct ipic_csr_mem_base
{
  unsigned char upper; /* bits a24-a31 mask */
  unsigned char lower; /* bits a16-a23 mask */
};
typedef struct ipic_csr_mem_base IPIC_CSR_MEM_BASE;

struct ipic_csr_int_ctrl
{
  unsigned char int0; /* bits a24-a31 mask */
  unsigned char int1; /* bits a16-a23 mask */
};
typedef struct ipic_csr_int_ctrl IPIC_CSR_INT_CNTL;

struct ipic_ctrl_stat_regs
{
  unsigned char chipId;
  unsigned char chipRevision;
  unsigned char reserved1[2];
  IPIC_CSR_MEM_BASE memBase[IPIC_MAX_SITES];
  unsigned char memSize[IPIC_MAX_SITES]; /*bits 16-23 0=64KB,1=128KB,3=256KB,7=512kb...*/
  IPIC_CSR_INT_CNTL intCntl[IPIC_MAX_SITES];
  unsigned char genCntl[IPIC_MAX_SITES];
  unsigned char reserved2[3];
  unsigned char ipReset;
};
typedef struct ipic_ctrl_stat_regs IPIC_CNTL_STAT_REGS;

/* there exists IPIC_MAX_SITES of the IPIC_MEMORY structures starting */
struct ipic_memory
{
  char ipIoSpace[128];
  IP_ID_PROM ipIdSpace; /* 64 bytes space */
  char ipGunkSpace[64];
};
typedef struct ipic_memory IPIC_MEMORY;


struct ip_mem_block
{
  unsigned short* start;
  unsigned long length;
};
typedef struct ip_mem_block IP_MEM_BLOCK;

class IndustryPackSiteMVME162;

class IndustryPackCarrierMVME162 : public IndustryPackCarrier
{
public:
  IndustryPackCarrierMVME162(const char *carrierName, unsigned long memBaseIP);
  //  void report();  
  
  // control functions
  void reset();
  
  volatile void* allocMemMapIP(int size);  
  const char* getSiteName(int site);
  volatile IPIC_CNTL_STAT_REGS *getCSR();
  volatile IP_ID_PROM *getSiteMemBaseID(int site);
  volatile void* getSiteMemBaseIO(int site);
  bool probeSite (int site);

 private:
  unsigned long memBaseIP;
  unsigned long memUsedIP;
  unsigned long memNextIP;
  volatile IPIC_MEMORY* ipicMemory;
  volatile IPIC_CNTL_STAT_REGS* ipicCSR;
  friend class IndustryPackCarrierSiteMVME162;
};

class IndustryPackSiteMVME162 : public IndustryPackSite
{
public:
  IndustryPackSiteMVME162(IndustryPackCarrierMVME162 *pCarrier, int siteIndex);
  
  // control functions
  
  void memSpaceOn();
  void memSpaceOff();
  void dataWidth(int width);	// valid widths are 32,8,16
				     
  void recoveryTimer(int microseconds); // valid times are 0,2,4,8 us
  int errStatus();
				     
  // interrupt control functions valid num: 0=IRQ0, 1=IRQ1
  void intLevelSensitive(int num);
  void intEdgeSensitive(int num);
  void intLowLevelTriggered(int num);
  void intHighLevelTriggered(int num);
  void intFallingEdgeTriggered(int num);
  void intRisingEdgeTriggered(int num);
  void intSetLevel(int num, int level); // level=0-7
  void intClear(int num);
  void intClear(void);
  void intEnable(int num);
  void intDisable(int num);
  int intStatus(int num);
  int intIsEnabled(int num);
  
  void reset();
  
  volatile void* allocMemMapIP(int size);
  int intConfig(int num);
  volatile IP_ID_PROM* getMemBaseID();
  volatile void* getMemBaseIO();
  volatile void* getMemBaseIP();
 
 private:
  int csrIndex;
  volatile IPIC_CNTL_STAT_REGS* ipicCSR;
  volatile IP_ID_PROM* pMemBaseID;
  volatile void *pMemBaseIO;
  volatile void *pMemBaseIP;

  int flag; // user flag 0=not set, 1=set
  unsigned short lock; // lock for flag
  unsigned short intLock;
  friend class IndustryPackCarrierMVME162;
};

int initIndustryPackMVME162(const char *carrierName, unsigned long memBase)
{
  // make sure controller not already initialized
  IndustryPackCarrier *pIPC = IndustryPackCarrier::find(carrierName);
  if (pIPC) {
    printf("initIndustryPackMVME162 already initialized %s\n",carrierName);
    return(0);
  }
  new IndustryPackCarrierMVME162(carrierName,memBase);
  //  pCarrier->report();
  return (0);
}

IndustryPackCarrierMVME162::IndustryPackCarrierMVME162
(const char *name, unsigned long memBase)
{
  //  printf("IndustryPackCarrierMVME162::IndustryPackCarrierMVME162 name %s\n",name);
  registerName(name);

  ipicCSR = (IPIC_CNTL_STAT_REGS*)IPIC_CNTL_STAT_REGS_MAP;
  memUsedIP = 0;
  memBaseIP = memBase;
  memNextIP = memBase;
  
  reset();
  
  for (int i=0; i<IPIC_MAX_SITES; i++)
    {
      //      printf ("IndustryPackCarrierMVME162::IndustryPackCarrierMVME162 probing site %d\n",i);
      bool probe = probeSite(i);
      //      printf ("IndustryPackCarrierMVME162::IndustryPackCarrierMVME162 probe result %d\n",probe);

      if (probe)
	{
	  IndustryPackSiteMVME162 *pSite = new IndustryPackSiteMVME162(this,i);
	  addSite(pSite);
	}
    }

    return;
}


volatile IP_ID_PROM *IndustryPackCarrierMVME162::getSiteMemBaseID(int site)
{
  IP_ID_PROM *pMemBaseID = 0;
  switch (site)
    {
    case 0:
      pMemBaseID = (IP_ID_PROM *)IPIC_IP_A_ID_MAP;
      break;
    case 1:
      pMemBaseID = (IP_ID_PROM *)IPIC_IP_B_ID_MAP;
      break;
    case 2:
      pMemBaseID = (IP_ID_PROM *)IPIC_IP_C_ID_MAP;
      break;
    case 3:
      pMemBaseID = (IP_ID_PROM *)IPIC_IP_D_ID_MAP;
      break;
    default:
      break;
    }
  return (pMemBaseID);
}
 
volatile void *IndustryPackCarrierMVME162::getSiteMemBaseIO(int site)
{
  void *pMemBaseIO = 0;
  switch (site)
    {
    case 0:
      pMemBaseIO = (void *)IPIC_IP_A_IO_MAP;
      break;
    case 1:
      pMemBaseIO = (void *)IPIC_IP_B_IO_MAP;
      break;
    case 2:
      pMemBaseIO = (void *)IPIC_IP_C_IO_MAP;
      break;
    case 3:
      pMemBaseIO = (void *)IPIC_IP_D_IO_MAP;
      break;
    default:
      break;
    }
  return (pMemBaseIO);
}
 
const char *IndustryPackCarrierMVME162::getSiteName(int site)
{
  const char *pSiteName = 0;
  switch (site)
    {
    case 0:
      pSiteName = IPIC_IP_A_NAME;
      break;
    case 1:
      pSiteName = IPIC_IP_B_NAME;
      break;
    case 2:
      pSiteName = IPIC_IP_C_NAME;
      break;
    case 3:
      pSiteName = IPIC_IP_D_NAME;
      break;
    default:
      break;
    }
  return (pSiteName);
}

bool IndustryPackCarrierMVME162::probeSite(int site)
{

  volatile IP_ID_PROM *pProm = getSiteMemBaseID(site);
  //  printf("IndustryPackCarrierMVME162::probeSite site %d pProm %lx\n",site,(long)pProm);
  
  char val1 = 0;
  char val2 = 0;
  char val3 = 0;
  char val4 = 0;
  STATUS status1 = vxMemProbe ((char*) &(pProm->ascii1), VX_READ, 1, (char*)&val1);
  STATUS status2 = vxMemProbe ((char*) &(pProm->ascii2), VX_READ, 1, (char*)&val2);
  STATUS status3 = vxMemProbe ((char*) &(pProm->ascii3), VX_READ, 1, (char*)&val3);
  STATUS status4 = vxMemProbe ((char*) &(pProm->ascii4), VX_READ, 1, (char*)&val4);
  
  //  printf ("IndustryPackCarrierMVME162::probeSite Status %d %d %d %d\n",status1,status2,status3,status4);

  //   printf ("IndustryPackCarrierMVME162::probeSite val .%x.%x.%x.%x.\n",val1,val2,val3,val4);

  if ((status1==OK) && (status2==OK) && (status3==OK) && (status4==OK) &&
      (val1=='I') && (val2=='P') && (val3=='A') && (val4=='C'))
    {
      //      printf("IndustryPackCarrierMVME162::probeSite %d occupied\n",site);
      return(TRUE);
    }
  else
    {
      //      printf("IndustryPackCarrierMVME162::probeSite %d empty\n",site);
      return(FALSE);
    }
}

volatile void *IndustryPackCarrierMVME162::allocMemMapIP (int size)
{
  unsigned long start;
  unsigned long length;

  //  printf ("IndustryPackCarrierMVME162::allocMemMapIP memNextIP %lx\n",memNextIP);
  
  if (size>0)
    {
      length = (unsigned long)size;
      //      printf ("IndustryPackCarrieMVME162::allocMemMapIP length %lx\n",length);
      start = (unsigned long)memNextIP / length * length;
      //      printf ("IndustryPackCarrierMVME162::allocMemMapIP start %lx\n",start);     
      if (start != (unsigned long)memNextIP) start += length;
      //      printf ("IndustryPackCarrierMVME162::allocMemMapIP start %lx\n",start);
      memNextIP = start + length;
      //      printf ("IndustryPackCarrierMVME162::allocMemMapIP memNextIP %lx\n",memNextIP);
    }
  else
    {
      start=0;
      length=0;
      //      printf ("IndustryPackCarrierMVME162::allocMemMapIP start %lx\n",start);
      //      printf ("IndustryPackCarrierMVME162::allocMemMapIP length %lx\n",length);
    }
  
  //  printf("IndustryPackCarrierMVME162::allocMemMapIP start=%8.8x, length=%d\n", (int)start, (int)length);

  return ((volatile void *)start);
}

volatile IPIC_CNTL_STAT_REGS* IndustryPackCarrierMVME162::getCSR()
{
  return ipicCSR;
}

void IndustryPackCarrierMVME162::reset()
{ 
  ipicCSR->ipReset |= IP_RESET;
  for (volatile unsigned long del=0; del < 30000; del++);
  ipicCSR->ipReset &= ~IP_RESET;
}

IndustryPackSiteMVME162::IndustryPackSiteMVME162
(IndustryPackCarrierMVME162 *pCarrier, int site)
{
  csrIndex = site;
  pIpCarrier = pCarrier;
  ipicCSR = pCarrier->getCSR();
  siteName = pCarrier->getSiteName(site);
  pMemBaseID = pCarrier->getSiteMemBaseID(site);
  pMemBaseIO = pCarrier->getSiteMemBaseIO(site);
  pMemBaseIP = 0;

  //  printf("IndustryPackSiteMVME162::IndustryPackSiteMVME162 csrIndex %d\n",csrIndex);
  //  printf("IndustryPackSiteMVME162::IndustryPackSiteMVME162 ipicCSR %p\n",ipicCSR);
  //  printf("IndustryPackSiteMVME162::IndustryPackSiteMVME162 siteName %s\n",siteName);
  //  printf("IndustryPackSiteMVME162::IndustryPackSiteMVME162 pMemBaseID %p\n",pMemBaseID);
  //  printf("IndustryPackSiteMVME162::IndustryPackSiteMVME162 pMemBaseIO %p\n",pMemBaseIO);
  //  printf("IndustryPackSiteMVME162::IndustryPackSiteMVME162 pMemBaseIP %p\n",pMemBaseIP);

  printf("IndustryPackSiteMVME162:: site %s occupied by manufacturer %x model %x\n",
	 siteName,(int)pMemBaseID->manufacturer,(int)pMemBaseID->model);


  return;
}

volatile IP_ID_PROM *IndustryPackSiteMVME162::getMemBaseID()
{
  return (pMemBaseID);
}

volatile void * IndustryPackSiteMVME162::getMemBaseIO()
{
  return (pMemBaseIO);
}

volatile void * IndustryPackSiteMVME162::getMemBaseIP()
{
  return (pMemBaseIP);
}

void IndustryPackSiteMVME162::dataWidth(int width)
{
  switch (width)
    {
    case 32:
      ipicCSR->genCntl[csrIndex] &= ~CNTL_WIDTH;
      ipicCSR->genCntl[csrIndex] |= CNTL_WIDTH_32;
      break;
    case 16:
      ipicCSR->genCntl[csrIndex] &= ~CNTL_WIDTH;
      ipicCSR->genCntl[csrIndex] |= CNTL_WIDTH_16;
      break;
    case 8:
      ipicCSR->genCntl[csrIndex] &= ~CNTL_WIDTH;
      ipicCSR->genCntl[csrIndex] |= CNTL_WIDTH_8;
      break;
    default:
      break;
    }
}

void IndustryPackSiteMVME162::recoveryTimer(int us)
{
  switch (us)
    {
    case 0:
      ipicCSR->genCntl[csrIndex] &= ~CNTL_RT;
      ipicCSR->genCntl[csrIndex] |= CNTL_RT_0;
      break;
    case 2:
      ipicCSR->genCntl[csrIndex] &= ~CNTL_RT;
      ipicCSR->genCntl[csrIndex] |= CNTL_RT_2;
      break;
    case 4:
      ipicCSR->genCntl[csrIndex] &= ~CNTL_RT;
      ipicCSR->genCntl[csrIndex] |= CNTL_RT_4;
      break;
    case 8:
      ipicCSR->genCntl[csrIndex] &= ~CNTL_RT;
      ipicCSR->genCntl[csrIndex] |= CNTL_RT_8;
      break;
    default:
      break;
    }
}

void IndustryPackSiteMVME162::intDisable(int num)
{
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 &= ~IP_INT_IEN;
  else		ipicCSR->intCntl[csrIndex].int1 &= ~IP_INT_IEN;
}

void IndustryPackSiteMVME162::intEnable(int num)
{
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 |= IP_INT_IEN;
  else		ipicCSR->intCntl[csrIndex].int1 |= IP_INT_IEN;
}

void IndustryPackSiteMVME162::intClear(int num)
{
  if (num==0)	ipicCSR->intCntl[csrIndex].int0 |= IP_INT_ICLR;
  else		ipicCSR->intCntl[csrIndex].int1 |= IP_INT_ICLR;
}

void IndustryPackSiteMVME162::intClear(void)
{
  if (ipicCSR->intCntl[csrIndex].int0 & IP_INT_INT)
    ipicCSR->intCntl[csrIndex].int0 |= IP_INT_ICLR;
  
  if (ipicCSR->intCntl[csrIndex].int1 & IP_INT_INT)
    ipicCSR->intCntl[csrIndex].int1 |= IP_INT_ICLR;
}

void IndustryPackSiteMVME162::intLevelSensitive(int num) 
{
  int f = 0;
  
  if (intIsEnabled(num)) { f=1; intDisable(num); }
  
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 &= ~IP_INT_EL;
  else		ipicCSR->intCntl[csrIndex].int1 &= ~IP_INT_EL;
  
  intClear(num);
  if (f) intEnable(num);
}

void IndustryPackSiteMVME162::intLowLevelTriggered(int num)
{
  int f = 0;
  
  if (intIsEnabled(num)) { f=1; intDisable(num); }
  
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 &= ~IP_INT_PLTY;
  else		ipicCSR->intCntl[csrIndex].int1 &= ~IP_INT_PLTY;
  
  intClear(num);
  if (f) intEnable(num);
}

void IndustryPackSiteMVME162::intHighLevelTriggered(int num)
{
  int f = 0;
  
  if (intIsEnabled(num)) { f=1; intDisable(num); }
  
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 |= IP_INT_PLTY;
  else		ipicCSR->intCntl[csrIndex].int1 |= IP_INT_PLTY;
  
  intClear(num);
  if (f) intEnable(num);
}

void IndustryPackSiteMVME162::intEdgeSensitive(int num) 
{
  int f = 0;
  
  if (intIsEnabled(num)) { f=1; intDisable(num); }
  
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 |= IP_INT_EL;
  else		ipicCSR->intCntl[csrIndex].int1 |= IP_INT_EL;
  
  intClear(num);
  if (f) intEnable(num);
}

void IndustryPackSiteMVME162::intFallingEdgeTriggered(int num) 
{
  int f = 0;
  
  if (intIsEnabled(num)) { f=1; intDisable(num); }
  
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 &= ~IP_INT_PLTY;
  else		ipicCSR->intCntl[csrIndex].int1 &= ~IP_INT_PLTY;
  
  intClear(num);
  if (f) intEnable(num);
}

void IndustryPackSiteMVME162::intRisingEdgeTriggered(int num) 
{
  int f = 0;
  
  if (intIsEnabled(num)) { f=1; intDisable(num); }
  
  if (num == 0)	ipicCSR->intCntl[csrIndex].int0 |= IP_INT_PLTY;
  else		ipicCSR->intCntl[csrIndex].int1 |= IP_INT_PLTY;
  
  intClear(num);
  if (f) intEnable(num);
}

void IndustryPackSiteMVME162::intSetLevel(int num, int level)
{
  unsigned char l;
  
  if (level > 7) return;
  l = (unsigned char)level;
  
  if (num == 0)
    {
      ipicCSR->intCntl[csrIndex].int0 &= ~IP_INT_LEVEL;
      ipicCSR->intCntl[csrIndex].int0 |= l;
    }
  else
    {
      ipicCSR->intCntl[csrIndex].int1 &= ~IP_INT_LEVEL;
      ipicCSR->intCntl[csrIndex].int1 |= l;
    }
}

int IndustryPackSiteMVME162::intStatus(int num)
{
  int i;
  
  if (num == 0)	i = ipicCSR->intCntl[csrIndex].int0 & IP_INT_INT;
  else		i = ipicCSR->intCntl[csrIndex].int1 & IP_INT_INT;
  
  return i ? 1 : 0;
}

int IndustryPackSiteMVME162::intIsEnabled(int num)
{
  unsigned char x;
  
  if (num == 0)	x = ipicCSR->intCntl[csrIndex].int0 & IP_INT_IEN;
  else		x = ipicCSR->intCntl[csrIndex].int1 & IP_INT_IEN;
  
  return x ? 1 : 0;
}

volatile void *IndustryPackSiteMVME162::allocMemMapIP(int size)
{
  unsigned long base = (unsigned long) pIpCarrier->allocMemMapIP(size);
  
  // printf ("IndustryPackModuleMVME162::allocMemMapIP size %lx base %lx\n", size, base);

  // control reg - memory access off
  ipicCSR->intCntl[csrIndex].int0 = IP_INT_ICLR|IP_INT_EL|IP_INT_PLTY;
  ipicCSR->intCntl[csrIndex].int1 = IP_INT_ICLR|IP_INT_EL|IP_INT_PLTY;
  
  // allocate a chuck of d32 ram if needed
  if (size > 0)
    {
      ipicCSR->memBase[csrIndex].upper = (unsigned char)(base>>24);
      ipicCSR->memBase[csrIndex].lower = (unsigned char)((base&0x00ff0000)>>16);
      ipicCSR->memSize[csrIndex] = (unsigned char)((size-1)>>16);
      
      ipicCSR->genCntl[csrIndex] = CNTL_WIDTH_16|CNTL_RT_8|CNTL_MEN;
    }
  else
    ipicCSR->genCntl[csrIndex] = CNTL_WIDTH_16|CNTL_RT_8;
  
  return ((volatile void *)base);
}

int IndustryPackSiteMVME162::intConfig(int num)
{
    intDisable(num);
    intSetLevel(num,3);
    intLevelSensitive(num);
    intLowLevelTriggered(num);
    intClear(num);
    return(0);
}


void IndustryPackSiteMVME162::memSpaceOn() { ipicCSR->genCntl[csrIndex] |= CNTL_MEN; }
void IndustryPackSiteMVME162::memSpaceOff() { ipicCSR->genCntl[csrIndex] &= ~CNTL_MEN; }
int IndustryPackSiteMVME162::errStatus() { return ipicCSR->genCntl[csrIndex] & CNTL_ERR ? 1 : 0; }

