//serialPortPeek.cc

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
/*
	Author: Marty Kraimer
	Date: 21OCT98
*/
extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
 
#include <tickLib.h>
#include <taskLib.h>
#include <rngLib.h>
}
 
int ringBufSize = 1000;

#include "SerialPort.h"
 
class PeekData {
public:
    bool isReceive;
    unsigned char byte;
};

class PeekPrivate {
public:
    RING_ID ringbuf;
};

static void peakTask(const char *portName,int nseconds);
static char taskname[] = "serialPeek";

int serialPortPeek(const char *portName,int nseconds)
{
    taskSpawn(taskname,130,VX_FP_TASK,2000,
	(FUNCPTR)peakTask,
	(int)portName,nseconds,0,0,0,0,0,0,0,0);
    return(0);
}

static void peakHandler(void* parm,bool isReceive, unsigned char byte)
{
    PeekPrivate *pvt = (PeekPrivate *)parm;
    PeekData data;
    data.isReceive = isReceive;
    data.byte = byte;
    rngBufPut(pvt->ringbuf,(char *)&data,sizeof(PeekData));
}

static void printQueue(bool *isReceiveQueue, unsigned char *byteQueue,int len)
{
    for(int i=0;i<len; i++) 
	printf("%c%2.2x ",(isReceiveQueue[i] ? 'R' : 'T'),byteQueue[i]);
    for(int i=0;i<len; i++)
	printf("%c", (isprint(byteQueue[i]) ? byteQueue[i] : '.'));
    printf("\n");
}
    
static void peakTask(const char *portName,int nseconds)
{
    PeekPrivate pvt;
    bool isReceiveQueue[10];
    unsigned char byteQueue[10];
    int nextQueue = 0;

    if((pvt.ringbuf = rngCreate(sizeof(void *) * ringBufSize))==NULL){
	printf("peakTask: initWatchDog failed\n");
	return;
    }
    if(!SerialPort::setPeekHandler(portName,&pvt,peakHandler)) return;
    unsigned int start = (unsigned int )::tickGet();
    printf("\n");
    while(((unsigned int)tickGet() - start)
    < (unsigned int)(nseconds*sysClkRateGet())) {
	taskDelay(sysClkRateGet());
	while (rngNBytes(pvt.ringbuf)>=(int)sizeof(PeekData)){
	    PeekData data;
	    if(rngBufGet(pvt.ringbuf,(char *)&data,
	    sizeof(PeekData))!=(int)sizeof(PeekData)) {
		printf("peakTask rngBufGet returned error\n");
	    } else {
		isReceiveQueue[nextQueue] = data.isReceive;
		byteQueue[nextQueue] = data.byte;
		if(++nextQueue == 10) {
		    printQueue(isReceiveQueue,byteQueue,nextQueue);
		    nextQueue = 0;
		}
	    }
	}
    }
    if(nextQueue>0) printQueue(isReceiveQueue,byteQueue,nextQueue);
    printf("serialPortPeek done\n");
    SerialPort::freePeekHandler(portName);
    rngDelete(pvt.ringbuf);
}
