//devLove.cc
// Author: Jim Kowalkowski
// Revised: 2/15/95
// Changes for MPF: Marty Kraimer
// Date: 27OCT98
extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
#include <math.h>
 
#include "dbAccess.h"
#include "dbDefs.h"
#include "dbScan.h"
#include "link.h"
#include "epicsPrint.h"
#include "dbCommon.h"
#include "aiRecord.h"
#include "recSup.h"
}

#include "Message.h"
#include "Int32Message.h"
#include "DevMpf.h"
#include "loveServer.h"

// INP has form Cx Sx @address

enum getState {getStart, getDecimalPlaces, getData};

class DevLove : public DevMpf
{
public:
    DevLove(dbCommon*,DBLINK*);
    long startIO(dbCommon* pr);
    long completeIO(dbCommon* pr,Message* m);
    static long dev_init(void*);
private:
    int32 address;
    double cvtFactor;
    getState state;
};

MAKE_DSET(devAiLove,DevLove::dev_init)

long DevLove::dev_init(void* v)
{
    aiRecord* pr = (aiRecord*)v;
    new DevLove((dbCommon*)pr,&(pr->inp));
    return(0);
}

DevLove::DevLove(dbCommon* pr,DBLINK* l)
: DevMpf(pr,l,false), address(0), cvtFactor(1.0) , state(getStart)
{
    const char* pv = getUserParm();
    if(pv==0) {
	epicsPrintf("%s DevLove Illegal INP field\n",pr->name);
        pr->pact = TRUE;
        return;
    }
    ::sscanf(pv,"%x",&address);
}

long DevLove::startIO(dbCommon *pr)
{
    Int32Message *message = new Int32Message;
    if(state==getStart) {
        message->cmd = R_DPT;
        state = getDecimalPlaces;
    } else if(state==getData) {
        message->cmd = R_PV;
    } else {
	epicsPrintf("%s DevLove illegal state in startIO\n",pr->name);
	delete message;
        return(-1);
    }
    message->address = address;
    message->timeout = 20;
    return(sendReply(message));
}

long DevLove::completeIO(dbCommon* pr,Message* pmessage)
{
    aiRecord* precord = (aiRecord*)pr;
    if((pmessage->getType()) != messageTypeInt32) {
	state = getStart;
	epicsPrintf("%s DevLove::completeIO illegal message.\n", precord->name);
	recGblSetSevr(precord,READ_ALARM,INVALID_ALARM);
	return(0);
    }
    Int32Message *pInt32Message = (Int32Message *)pmessage;
    if(pInt32Message->status!=0) {
        recGblSetSevr(precord,READ_ALARM,INVALID_ALARM);
        state = getStart;
    } else if(state==getDecimalPlaces) {
        double decimal_places = (double)pInt32Message->value;
        cvtFactor = 1.0/pow(10.0,decimal_places);
        state = getData;
        scanOnce((void *)precord);
    } else if(state==getData) {
        precord->val = ((double)pInt32Message->value) * cvtFactor;
        precord->udf = 0;
    } else {
        recGblSetSevr(precord,READ_ALARM,INVALID_ALARM);
        state = getStart;
	epicsPrintf("%s DevLove illegal state in completeIO\n",pr->name);
    }
    delete pInt32Message;
    return(MPF_NoConvert);
}
