// localRouter.cc

/* Author:  Marty Kraimer Date:    31MAR1998 */

/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/

extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
}

#include "Message.h"
#include "MessagePvt.h"
#include "ConnectMessage.h"
#include "WatchDog.h"

static int secondsBetweenCalc = 10;

extern "C" int localMessageRouterStart(int location);

class LMRServer;

struct Statistics
{
    int nNew;
    int nPerSecond;
    WatchDog *wdId;
};

class LMRClient: public MessageRouterClient
{
public:
    LMRClient(int location);
    virtual void bind(const char *server,MessageClient *messageClient);
    virtual int send(Message *message);
    virtual void report();
    LMRServer *pLMRServer;
    // statistics
    Statistics *pstats;
};

class LMRServer: public MessageRouterServer
{
public:
    LMRServer(int location);
    virtual int reply(Message *);
    virtual void report();
    LMRClient *pLMRClient;
    // statistics
    Statistics *pstats;
};

int localMessageRouterStart(int location)
{
    LMRClient *pLMRClient;
    LMRServer *pLMRServer;

    pLMRClient = new LMRClient(location);
    pLMRServer = new LMRServer(location);
    pLMRClient->pLMRServer = pLMRServer;
    pLMRServer->pLMRClient = pLMRClient;
    return(0);
}

static void calculateStats(void *pvt)
{
    Statistics *pstat = (Statistics *)pvt;
    int nNew;

    nNew = pstat->nNew; pstat->nNew = 0;
    pstat->nPerSecond = nNew/secondsBetweenCalc;
    pstat->wdId->start(secondsBetweenCalc,calculateStats,(void *)pstat);
}

LMRClient::LMRClient(int location) : MessageRouterClient(location)
{
    pstats = new Statistics;
    pstats->nNew = 0;
    pstats->nPerSecond = 0;
    pstats->wdId = new WatchDog;
    pstats->wdId->start(secondsBetweenCalc,calculateStats,(void *)pstats);
    prouter->registerClientRouter(this,location);
}

void LMRClient::bind(
    const char *server,MessageClient *pMessageClient)
{
    MessageServer *pMessageServer;
    ConnectMessage *pmessage;
    MessagePvt     *msgPvt;

    pMessageServer = prouter->findMessageServer(server);
    pmessage = new ConnectMessage;
    msgPvt = getMessagePvt((Message *)pmessage);
    msgPvt->pMessageRouterClient = this;
    if(pMessageServer) {
	msgPvt->pMessageClient = pMessageClient;
	msgPvt->pMessageRouterServer = pLMRServer;
	msgPvt->pMessageServer = pMessageServer;
	pmessage->status = connectYes;
    } else {
	pmessage->status = connectNoServer;
    }
    pMessageClient->bindCallback(pmessage);
}

int LMRClient::send(Message *message)
{
    MessagePvt *msgPvt = getMessagePvt(message);
    MessageServer *pMessageServer = msgPvt->pMessageServer;

    pstats->nNew++;
    pMessageServer->queueMessage(message);
    return(0);
}

void LMRClient::report()
{
    printf(" %d LMRClient\n",getLocation());
    printf("   sendPerSec %d\n",pstats->nPerSecond);
}

LMRServer::LMRServer(int location) : MessageRouterServer(location)
{
    pstats = new Statistics;
    pstats->nNew = 0;
    pstats->nPerSecond = 0;
    pstats->wdId = new WatchDog;
    pstats->wdId->start(secondsBetweenCalc,calculateStats,(void *)pstats);
    prouter->registerServerRouter(this,getLocation());
}

int LMRServer:: reply(Message *pmessage)
{
    MessagePvt *msgPvt = getMessagePvt(pmessage);
    MessageClient *pMessageClient = msgPvt->pMessageClient;

    pstats->nNew++;
    pMessageClient->clientCall(pmessage,pMessageClient->getClientPvt());
    return(0);
}

void LMRServer::report()
{
    printf(" %d LMRServer\n",getLocation());
    printf("   replyPerSec %d\n",pstats->nPerSecond);
}
