// gpibServer.c

// Author: Marty Kraimer 
// Date: 7/98


extern "C" {
#include <vxWorks.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
#include <taskLib.h>
#include <semLib.h>
}
 
#include "Message.h"
#include "Int32Message.h"
#include "Char8ArrayMessage.h"
#include "Gpib.h"

class GpibServer
{
public:
    GpibServer(const char *name,int card,int queueSize);
    static void gpibServer(GpibServer *);
    bool getStartOk() const {return(startOk);}
private:
    MessageServer *pMessageServer;
    Gpib* pGpib;
    bool startOk;
};
enum gpibCommand {gpibRead, gpibWrite, gpibWriteRead, gpibCmd};

static char taskname[] = "gpibServer";
int initGpibServer(char *serverName,int card, int queueSize)
{
    GpibServer *psg = new GpibServer(serverName,card,queueSize);
    if(!psg->getStartOk()) return(0);
    int taskId = taskSpawn(taskname,100,VX_FP_TASK,2000,
    (FUNCPTR)GpibServer::gpibServer,(int)psg,0,0,0,0,0,0,0,0,0);
    if(taskId==ERROR) 
    printf("gpibServer taskSpawn Failure\n");
    return(0);
}

GpibServer:: GpibServer(const char *name,int card,int queueSize)
: startOk(false)
{
    pGpib = Gpib::findGpib(card);
    if(pGpib==0) {
        printf("gpibServer: could not find Gpib\n");
        return;
    }
    pMessageServer = new MessageServer(name,queueSize);
    startOk = true;
}

void GpibServer::gpibServer(GpibServer *pgs)
{
    while(true) {
        pgs->pMessageServer->waitForMessage();
        Message *message;
        while((message = pgs->pMessageServer->receive())) {
	    if(message->getType()==messageTypeChar8Array) {
		Char8ArrayMessage  *pstring = (Char8ArrayMessage *)message;
		if(pstring->cmd==gpibCmd || pstring->cmd==gpibWrite) {
		    int status=0;
		    if(pstring->cmd==gpibWrite) {
		        status = pgs->pGpib-> write(pstring->value,
			    pstring->getSize(),pstring->address,
			    pstring->timeout);
		    } else {
		        status = pgs->pGpib->cmd(pstring->value,
			    pstring->getSize(), pstring->timeout);
		    }
		    Int32Message *reply = (Int32Message *)pgs->pMessageServer->
			allocReplyMessage(pstring,messageTypeInt32);
		    reply->status = status;
		    pgs->pMessageServer->reply(reply);
		} else {
		    printf("GpibServer illegal msg type %d\n",message->getType());
		}
		delete pstring;
	    } else if(message->getType()==messageTypeInt32) {
		Int32Message *int32Message = (Int32Message *)message;
		int status = 0;
		if(int32Message->cmd!=gpibRead
		&& int32Message->cmd!=gpibWriteRead) {
		    printf("GpibServer illegal cmd%d\n",int32Message->cmd);
		    delete int32Message; continue;
		}
		Char8ArrayMessage  *reply = (Char8ArrayMessage *)
		    pgs->pMessageServer->
		    allocReplyMessage(int32Message,messageTypeChar8Array);
		reply->allocValue(int32Message->value);
		if(int32Message->cmd==gpibWriteRead) {
                    pgs->pMessageServer->waitForMessage();
		    message = pgs->pMessageServer->receive();
		    if(message==0 || message->getType()!=messageTypeChar8Array){
		        printf("GpibServer expected messageTypeChar8Array\n");
			delete int32Message; delete message; continue;
		    }
		    Char8ArrayMessage  *pstring = (Char8ArrayMessage *)message;
		    int size = pstring->getSize();
		    if(pstring->cmd!=gpibWriteRead) {
		         printf("GpibServer expected cmd gpibWriteRead\n");
			 delete int32Message; delete pstring; continue;
		    }
		    status = pgs->pGpib->write(pstring->value, size,
			pstring->address, pstring->timeout);
		    if(status==size) status = 0; else status = -1;
		    delete pstring;
		}
		if(status==0) {
		    int actual=0;
		    status = pgs->pGpib->read(reply->value,
			int32Message->value,&actual,int32Message->address,
			int32Message->timeout, int32Message->extra);
		    reply->setSize(actual);
		}
		reply->status = status;
		delete int32Message;
		pgs->pMessageServer->reply(reply);
	    } else {
		printf("GpibServer got illegal message type %d\n",
		    message->getType());
		delete message;
	    }
	}
    }
}
