// clientChar8Array.cc

extern "C" {
#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <semLib.h>
#include <tickLib.h>
#include <taskLib.h>
}

#include "Message.h"
#include "ConnectMessage.h"
#include "Char8ArrayMessage.h"
#include "DLList.h"
#include "serialServer.h"

int clientChar8ArrayDebug = 0;
int clientChar8ArrayTimeout = 5;

typedef struct pvtData {
    connectStatus connectState;
    char8	valueSent[512];
    char8	valueReceived[512];
    SEM_ID	sem;
    bool	timeout;
}pvtData;

static void myCallback(Message *message,void *clientPvt)
{
    pvtData *ppvtData = (pvtData *)clientPvt;
    
    switch(message->getType()) {
    case messageTypeConnect:
	{
	    ConnectMessage *pConnectMessage = (ConnectMessage *)message;

	    printf("clientChar8Array messageTypeConnect status %s\n",
		connectStatusString[pConnectMessage->status]);
	    ppvtData->connectState = pConnectMessage->status;
	    delete pConnectMessage;
	    semGive(ppvtData->sem);
	}
	break;
    case messageTypeChar8Array:
	{
            Char8ArrayMessage *pChar8ArrayMessage =
		(Char8ArrayMessage *)message;
	
	    ::strcpy(ppvtData->valueReceived,
		pChar8ArrayMessage->value);
	    if(clientChar8ArrayDebug>1) {
		char buffer[100];

		sprintf(buffer,"client value %s pChar8ArrayMessage %p\n",
		    ppvtData->valueReceived,pChar8ArrayMessage);
		printf("%s",buffer);
	    }
	    delete pChar8ArrayMessage;
	    if(strcmp(ppvtData->valueReceived,ppvtData->valueSent)==0){
		if(clientChar8ArrayDebug>1) printf("semGive\n");
		semGive(ppvtData->sem);
	    }
	}
	break;
    default:
	printf("got illegal response message\n");
	break;;
    }
}

static void timeoutHandler(void *pvt)
{
    pvtData *ppvtData = (pvtData *)pvt;
    ppvtData->timeout = true;
    semGive(ppvtData->sem);
}


int clientChar8Array(char *server,int slot,
    int32 clientChar8ArraySize,int delay)
{
    MessageClient	*pMessageClient;
    pvtData		*ppvtData = new pvtData;
    int			status;
    Char8ArrayMessage 	*message;

    if(clientChar8ArraySize<=0) clientChar8ArraySize=16;
    ppvtData->connectState = connectNo;
    ppvtData->valueReceived[0] = 0;
    ppvtData->valueSent[0] = 0;
    ppvtData->timeout = false;
    pMessageClient = new MessageClient(myCallback,ppvtData);
    ppvtData->sem  = semBCreate(SEM_Q_PRIORITY,SEM_EMPTY);
    status = pMessageClient->bind(server,slot);
    if(status) {
	printf("bind returned %d\n",status);
	return(status);
    }
    char8 nextValue = 'z';
    while(true) {
	if(ppvtData->connectState!=connectYes) {
	    semTake(ppvtData->sem,60);
	    continue;
	}
	if(ppvtData->timeout) {
	    printf("clientChar8Array timeout occured\n");
	    ppvtData->timeout = false;
	}
	int nwait=0;
	while(strcmp(ppvtData->valueReceived,ppvtData->valueSent)!=0){
	    if(nwait==0)printf("clientChar8Array send %s receive %s\n",
		ppvtData->valueSent,ppvtData->valueReceived);
	    semTake(ppvtData->sem,60);
	    if(nwait++ == 5) break;
	}
	if(strcmp(ppvtData->valueReceived,ppvtData->valueSent)==0){
	    nextValue += 1;
	    if(nextValue > 'z') nextValue = 'a';
	}
	message = new Char8ArrayMessage;
	message->allocValue(clientChar8ArraySize);
	message->setSize(clientChar8ArraySize);
	for(int i=0; i<(clientChar8ArraySize-1); i++)
	    message->value[i] = nextValue;
	message->value[clientChar8ArraySize-1] = 0;
	::strcpy(ppvtData->valueSent,message->value);
	if(clientChar8ArrayDebug) printf("sending %s\n",message->value);
	message->timeout=20;
	message->cmd = cmdWriteRead;
	status = pMessageClient->sendTimeout(message,
		clientChar8ArrayTimeout,timeoutHandler,(void*)ppvtData);
	if(status) printf("send returned %d\n",status);
	STATUS semStatus = semTake(ppvtData->sem,WAIT_FOREVER);
	if(semStatus==ERROR) printf("semTake timed out\n");
	if(delay>0) taskDelay(delay);
    }
    return(0);
}
