// DLList.cc

/* Author:  Marty Kraimer Date:    26MAR1998 */
 
/********************COPYRIGHT NOTIFICATION**********************************
This software was developed under a United States Government license
described on the COPYRIGHT_UniversityOfChicago file included as part
of this distribution.
****************************************************************************/
extern "C" {
#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdio.h>

#include <semLib.h>
}

#include "FreeList.h"
#include "DLList.h"

static FreeList *pFreeNodeList = 0;

DLList::DLList() {
    first=0;
    last=0;
    count=0;
    sem = semBCreate(SEM_Q_PRIORITY,SEM_FULL);
}

DLList::~DLList() {
    empty();
    semDelete(sem);
}

void DLList::addEnd(DLNode *node)
{
    if(first==0) {
	first = last = node;
	node->next = node->prev = 0;
    } else {
        node->next = 0; node->prev = last; last->next = node; last = node;
    }
    count++;
}

void DLList::addBegin(DLNode *node)
{
    if(first==0) {
	first = last = node;
	node->next = node->prev = 0;
    } else {
        node->next = first; node->prev = 0; first->prev = node; first = node;
    }
    count++;
}

void DLList::addAfter(DLNode *node,DLNode *newnode)
{
    newnode->next = node->next; newnode->prev = node;
    if(node->next==0) {
        last = newnode;
    } else {
        node->next->prev = newnode;
    }
    node->next = newnode;
    count++;
}

void DLList::addBefore(DLNode *node,DLNode *newnode)
{
    newnode->next = node; newnode->prev = node->prev;
    if(node->prev==0) {
        first = newnode;
    } else {
        node->prev->next = newnode;
    }
    node->prev = newnode;
    count++;
}

void DLList::empty()
{
    DLNode *node;
    DLNode *pnext;

    node = getFirst();
    while(node) {
	pnext = getNext(node);
	freeNode(node);
	node = pnext;
    }
}

void DLList::freeNode(DLNode *node)
{
    if(node->prev==0) {
        first = node->next;
    } else {
        node->prev->next = node->next;
    }
    if(node->next==0) {
        last = node->prev;
    } else {
        node->next->prev = node->prev;
    }
    pFreeNodeList->free((void *)node);
    count--;
}

DLNode *DLList::allocNode()
{
    if(pFreeNodeList==0) {
	pFreeNodeList = new FreeList((int)sizeof(DLNode), 100);
printf("DLList::allocNode allocating 100 new nodes\n");
    }
    return((DLNode *)pFreeNodeList->calloc());
}
